unit LDN161HardcopyOfFormattedTextU1;

{$mode objfpc}{$H+}

(*This is in (at least!) two parts...

The main point of this is to play with putting misc text on a panel,
   with different sizes, font-families, etc. And then to produce
   hardcopy of what is in the panel.

Producing the hardcopy is an extension of what is explained at...
  http://sheepdogguides.com/lut/lt3Nhardc-short.htm

To use this, you need the (free, "officially endorsed"(?)
  Printer4Lazarus" package in your IDE, as discussed
  at the above-mentioned lt3Nhardc-short.htm
*)

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls,
  Printers, PrintersDlgs{provided by Printer4Lazarus package},
  Clipbrd{only needed for the "copy URL to clipboard" frill};

const vers='10 Jan 21';

type

  { TLDN161HardcopyOfFormattedTextF1 }

  TLDN161HardcopyOfFormattedTextF1 = class(TForm)
    buChangePanelWidth: TButton;
    buQuit: TButton;
    buFillWithTextSeeSourcecode: TButton;
    buSendToPrinter: TButton;
    buCopyLazTut: TButton;
    ePanelWidth: TEdit;
    laTxtClickButtonToEffectChange: TLabel;
    laTxtTheseDoneExplanation: TLabel;
    laTxtPanelWidthEditBoxInstructions: TLabel;
    paDrawHere: TPanel;
    PrintDialog1: TPrintDialog;
    procedure buChangePanelWidthClick(Sender: TObject);
    procedure buFillWithTextSeeSourcecodeClick(Sender: TObject);
    procedure buQuitClick(Sender: TObject);
    procedure buSendToPrinterClick(Sender: TObject);
    procedure buCopyLazTutClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormShow(Sender: TObject);

  private

  public

  end;

var
  LDN161HardcopyOfFormattedTextF1: TLDN161HardcopyOfFormattedTextF1;

implementation

{$R *.lfm}

{ TLDN161HardcopyOfFormattedTextF1 }

procedure TLDN161HardcopyOfFormattedTextF1.FormCreate(Sender: TObject);
begin
  //Many of these could be set with the Object Inspector. I prefer
  //to do them here, so anything that is not in default setting is
  //easily noticed.
  paDrawHere.BevelOuter:=bvNone;
  paDrawHere.BorderStyle:=bsSingle;//Also done with Object inspector...
  paDrawHere.BorderWidth:=1;//(ditto)... to make extent of panel visible.
  ePanelWidth.Text:='300';
  paDrawHere.caption:='';
  paDrawHere.Color:=clWhite;
  //Tried to do here... but didn't result in any text on screen...
  //   buFillWithTextSeeSourcecodeClick(Self);
  //SEE the form's OnPaint event handler, "FormPaint".

  caption:='LDN161HardcopyOfFormattedText, version '+vers;
end;

procedure TLDN161HardcopyOfFormattedTextF1.FormActivate(Sender: TObject);
begin
  buChangePanelWidthClick(Self);
  //Tried to do here... but didn't result in any text on screen...
  //   buFillWithTextSeeSourcecodeClick(Self);
  //SEE the form's OnShow event handler, "FormShow".
end;

procedure TLDN161HardcopyOfFormattedTextF1.FormShow(Sender: TObject);
begin
  //This threw no errors, but didn't "work" if put in
  //either FormCreate or FormActivate or here...
  //buFillWithTextSeeSourcecodeClick(Self);
end;

procedure TLDN161HardcopyOfFormattedTextF1.FormPaint(Sender: TObject);
begin
  buFillWithTextSeeSourcecodeClick(Self);
end;

procedure TLDN161HardcopyOfFormattedTextF1.buChangePanelWidthClick(Sender: TObject);
begin
  //This is crude.
  //The panel should change size without the need for clicking a button,
  //and the program should handle non-numeric values in ePanelWidth more
  //elegantly. But it "works", even in this crude state!
  paDrawHere.width:=strtoint(ePanelWidth.text);
end;

procedure TLDN161HardcopyOfFormattedTextF1.buFillWithTextSeeSourcecodeClick(Sender: TObject
  );
begin
  with paDrawHere.canvas do begin
    //The followoing is done very CRUDELY... for which I make
    //  no apology. Yes, I would do much of this differently, it
    //  my object was to prepare some text for printing.
    //But my object is to show you HOW to print text. What text
    //  is in the panel when you say "Print that" is up to you!

    //By the way.. a TRichMemo object does not have a canvas.
    //So you can't JUST substitute a TRichMemo box for paDrawHere.
    //Sigh.
    //There "must be a way"... but I haven't solved that one... yet.

    TextOut(10,10,'ABCDE- in default pt size');
    Font.Size:=20;
    TextOut(8,40,'ABCDE- 20 pt 012345678b23456789c');
    Font.Size:=10;
    TextOut(10,80,'No harm arises if text');
    TextOut(10,100,'spills out of box... but');
    TextOut(10,120,'it');
    Font.Style:=[fsItalic,fsBold];
    TextOut(23,120,'does');
    Font.Style:=[];
    TextOut(57,120,'get lost.');
    end;//with...
end;//buFillWithTextSeeSourcecodeClick

procedure TLDN161HardcopyOfFormattedTextF1.buQuitClick(Sender: TObject);
begin
  application.terminate;
end;

procedure TLDN161HardcopyOfFormattedTextF1.buSendToPrinterClick(Sender: TObject);
var rectSource, rectDest: TRect;
  //The "PrintDialog1.execute" may be annoying, but if you remove it
  // the app will probably not run on some systems.
begin
  if PrintDialog1.execute then begin//To prevent SIGSEGV error
  with Printer do
  try
    BeginDoc;
    Canvas.Font.Name := 'Courier New';
    Canvas.Font.Size := 10;
    Canvas.Font.Color := clBlack;
    Canvas.TextOut(10,10,'xxx');
    //paDrawHere.PaintTo(Printer.Canvas, 0, 0);

    //copyRect: Copies something into a part of whatever object you
    //   have used. I.e. MyGreenCanvas.copyRect... would copy something
    //   TO (a part of) the canvas called MyGreenCanvas
    //Params...
    //1st says what rectangle to copy into
    //2nd says what canvas the coped area is to come from
    //3rd says what rectangle from source should be copied to dest, with
    //   automatic re-sizing implied and functioning.

    //See https://lazarus-ccr.sourceforge.io/docs/rtl/types/trect.html
    // for the various ways rectangles can be specified.

    rectSource.left:=0;
    rectSource.top:=0;
    rectSource.right:=paDrawHere.width;
    rectSource.bottom:=paDrawHere.height;

    rectDest.left:=0;
    rectDest.top:=0;
    rectDest.right:=paDrawHere.width*5; //Because printer has more dpi than monitor
    rectDest.bottom:=paDrawHere.width*5;

    canvas.copyRect(rectDest,paDrawHere.canvas,rectSource)

    //That copyRect will copy whatever is on the screen in the area
    //  specified...
    //If you use paDrawHere as the source, what is
    //  printed will even include, for example, a button that just
    //  happens to be "on top of" paDrawHere at the time.
    //You can, if you wish, use LDN161HardcopyOfFormattedTextF1 as
    //  the source! It has a perfectly usable canvas, too.

  finally
    EndDoc;
  end;//of "try.."
 end;//If Pri..
end;//buSendToPrinterClick

procedure TLDN161HardcopyOfFormattedTextF1.buCopyLazTutClick(Sender: TObject);
begin
  Clipboard.AsText:=('http://sheepdogguides.com/lut/lt3Nhardc-short.htm');
  showmessage('The URL is now in your clipboard. Got to browser, '+
     'paste, to go to help page.');
end;

end.

